;BENCH.SC

;==============================================================================
;
;  Written by Steve Brann, CIS: 74730,3223
;
;  NOTE: About 25MB of free space on your disk is required to run this
;  benchmark.
;
;  Program tests the effectivenss of disk I/O, cache, memory utilization etc.
;  I've found this program to be helpful in tuning memory and disk cache
;  configurations etc. Thus, you can make trade-off's based on the size of
;  your paradox tables, I/O activity, and system tuning.
;
;  (See Text - EndText below).
;
;  For example, I learned that even though I have a caching disk contoller
;  with 4MB, if I added software cache, my performance on this benchmark
;  improved about 8%.
;
;  The table named RESULTS can be analyzed to determine how performance
;  degrades as a function of the number of records in the tables.
;
;  To change the number of iterations, change the number of iterations by
;  altering the variable assignment for max.records on line 134.
;
;  To delete one or more of the three tables created, uncomment the delete
;  statement(s) on lines 116 thru 118.
;
;
;  My machine specifications are:
;
;  Insight 486/33 EISA with 32MB memory, 1GB SCSI drive with DTC
;  4MB caching controller.
;
;
;  I'm running Paradox with 15500K bytes of extended memory.
;  I added a 12MB software cache in expanded memory for this test.
;
;  My results:
;
;  Phase I   -  256.1 records per second            .1085 decimal hours
;  Phase II  -   56.7 records per second            .4901 decimal hours
;  Phase III -   36.4 records per second            .7631 decimal hours
;
;  Total time (HHHH:MM:SS.SS) =  0001:21:42.05  =  1.3617 decimal hours
;
;  Please feel free to E-mail me your results.
;
;==============================================================================


Proc bench()
   @0,0
   Clear Eos
   @0,0
   Text
   This program creates two tables, (TESTFILE and KEYFILE).  The program will
   generate 100,000 18 byte records in the TESTFILE table with a field
   containing a random 10 character alpha string that will become a key to
   the second table, KEYFILE which contains 150 byte records.  TESTFILE size
   will be about 1.8MB and KEYFILE size will be about 23MB including the index.

   PHASE 1:
      Develops a sequential table of 100,000 records (TESTFILE).
   PHASE 2:
      Processes the sequential table and creates a second keyed table (KEYFILE)
      with a longer record length.  Duplicates are checked for when the table
      is created, so it is possible that this table will have fewer records.
   PHASE 3:
      Processes the sequential table against the keyed table and does an update
      of the keyed table (read and re-write) by copying the first field in
      TESTFILE to the data portion of the keyed table.

   This test is set up to develop 100,000 records.  If you want to use
   fewer records, just edit the script, find the create.keys() proc and change
   the assignment of max.records to a different value.  The records per second
   for Phase 1 are fairly constant.  For Phases 2 and 3, snapshots of records
   per second are taken every 100 records and filed in a table named RESULTS.

   Press any key to perform the test, or "Q" to exit...
   Endtext
   Accept "A1" To x
   If upper(x) = "Q" Then
      Quit
   Endif
   Cursor Off
   @0,0
   Clear Eos
   ?? "Working..."

   Create "testfile"
   "Testkey"    : "A10",
   "Testcount"  : "N"

   Create "keyfile"
   "KeyKey"     : "A10*",
   "KeyCount"   : "N",
   "KeyData"    : "A132"


   Create "results"
   "Phase"             : "S",
   "RecordNbr"         : "N",
   "CumRecsPerSec"     : "N",
   "Last100RecsPerSec" : "N",
   "DecimalHrs"        : "N"

   View "results"
   View "testfile"
   View "keyfile"
   Coeditkey
   create.keys()
   sukspit()
   sukspit2()
   ;delete "testfile"
   ;delete "keyfile"
   ;delete "results"
Endproc




Proc create.keys()
   counter = 0
   Moveto "testfile"
   phase1.date.started = today()
   phase1.time.started = ticks()
   @0,0
   Clear Eos
   ;------------------------------------------------------------------
   ;  Change the assignment in the next line for fewer or more records
   ;------------------------------------------------------------------
   max.records = 100000
   For i From 1 To max.records
      counter = counter + 1
      Ins
      [testkey] = substr(format("w11,ez",rand()*1e10),2,10)
      [testcount] = counter
      If mod(counter,100) = 0 Then
         @0,0 ?? "Records created    = " , counter
         calc.elapsed(phase1.date.started,phase1.time.started,today(),ticks())
         @2,0
         Clear Eol
         phase1.records.per.sec = format("W7.1",counter / decimal.secs)
         @1,0 ?? "Records per second = " , phase1.records.per.sec
      Endif
   Endfor
   phase1.date.ended = today()
   phase1.time.ended = ticks()
   calc.elapsed(phase1.date.started,phase1.time.started,
   phase1.date.ended,phase1.time.ended)
   phase1.decimal.hours = decimal.hrs
   phase1.display.time = display.time
   @2,0
   Clear Eol
   @2,0 ?? "Total hours = " , format("W10.4,ecz",phase1.decimal.hours)
   @2,35 ?? "Phase 1 time (HHHH:MM:SS.SS) = " ,phase1.display.time
Endproc




Proc sukspit()
   @3,0 ?? fill("-",80)
   Cursor Off
   start.date = today()
   start.time = time()
   record.count = 0
   old.decimal.secs = 0
   Moveto "testfile"
   phase2.date.started = today()
   phase2.time.started = ticks()
   Scan
      record.count = record.count + 1
      If mod(record.count,100) = 0 Then
         @4,0
         Clear Eol
         @4,0 ?? "Records processed  = " , record.count
         calc.elapsed(phase2.date.started,phase2.time.started,today(),ticks())
         @5,0
         Clear Eol
         y = record.count / decimal.secs
         phase2.records.per.sec = format("W7.1",y)
         @5,0 ?? "Records per second = " , phase2.records.per.sec
         x = 100 / (decimal.secs - old.decimal.secs)
         Phase2.last.100.records = strval(format("W7.1",x))
         @5,35 ??"Last 100 records   =",phase2.last.100.records
         old.decimal.secs = decimal.secs
         Moveto "results"
         end
         down
         [Phase] = 2
         [RecordNbr] = record.count
         [CumRecsPerSec] = y
         [Last100RecsPerSec] = x
         [DecimalHrs] = decimal.hrs
         Moveto "Testfile"
      Endif
      Key = [testkey]
      Count = [testcount]
      Moveto "keyfile"
      Moveto Field "keykey"
      Locate Key
      If Not Retval Then
         Ins
         [keykey] = Key
         [keycount] = Count
      Endif
      Moveto "testfile"
      Moveto Field "testkey"
   Endscan
   phase2.date.ended = today()
   phase2.time.ended = ticks()
   calc.elapsed(phase2.date.started,phase2.time.started,
                phase2.date.ended,phase2.time.ended)
   phase2.decimal.hours = decimal.hrs
   phase2.display.time = display.time
   @6,0
   Clear Eol
   @6,0 ?? "Total hours = " , format("W10.4,ecz",phase2.decimal.hours)
   @6,35 ?? "Phase 2 time (HHHH:MM:SS.SS) = " ,phase2.display.time
   @7,0 ?? fill("-",80)
Endproc




Proc sukspit2()
   Cursor Off
   start.date = today()
   start.time = time()
   record.count = 0
   old.decimal.secs = 0
   Moveto "testfile"
   phase3.date.started = today()
   phase3.time.started = ticks()
   Scan
      record.count = record.count + 1
      If mod(record.count,100) = 0 Then
         @8,0
         Clear Eol
         @8,0 ?? "Records processed  = " , record.count
         calc.elapsed(phase3.date.started,phase3.time.started,today(),ticks())
         @9,0
         Clear Eol
         y = record.count / decimal.secs
         phase3.records.per.sec = format("W7.1",y)
         @9,0 ?? "Records per second = " , phase3.records.per.sec
         x = 100 / (decimal.secs - old.decimal.secs)
         Phase3.last.100.records = strval(format("W7.1",x))
         @9,35 ??"Last 100 records   =",phase3.last.100.records
         old.decimal.secs = decimal.secs
         Moveto "results"
         End
         Down
         [Phase] = 3
         [RecordNbr] = record.count
         [CumRecsPerSec] = y
         [Last100RecsPerSec] = x
         [DecimalHrs] = decimal.hrs
         Moveto "Testfile"
      Endif
      Key = [testkey]
      Count = [testcount]
      Moveto "keyfile"
      Moveto Field "keykey"
      Locate Key
      If Retval Then
         [Keydata] = strval(count)
      Endif
      Moveto "testfile"
   Endscan
   phase3.date.ended = today()
   phase3.time.ended = ticks()
   calc.elapsed(phase3.date.started,phase3.time.started,
                phase3.date.ended,phase3.time.ended)
   phase3.decimal.hours = decimal.hrs
   phase3.display.time = display.time
   @10,0
   Clear Eol
   @10,0 ?? "Total hours = " , format("W10.4,ecz",phase3.decimal.hours)
   @10,35 ?? "Phase 3 time (HHHH:MM:SS.SS) = " ,phase3.display.time
   @12,0 ?? fill("-",80)
   total.hours = phase1.decimal.hours+phase2.decimal.hours+phase3.decimal.hours
   @14,0??"Total hours = ", format("W10.4,ecz",total.hours)
   temp.fraction = mod(total.hours,1)
   temp.hrs      = int(total.hours)
   temp.mins     = int(temp.fraction * 60)
   temp.secs     = temp.fraction * 3600 - temp.mins * 60
   display.hrs   = substr(format("W5,ez",temp.hrs),2,4)
   display.mins  = substr(format("W3,ez",temp.mins),2,2)
   display.secs  = substr(format("W6.2,ez",temp.secs),2,5)
   display.time  = display.hrs+":"+display.mins+":"+display.secs
   @14,35?? "All phases (HHHH:MM:SS.SS)   = " , display.time
   Savevars All
   Do_It!
   Clearall
   @19,0
   Text
This concludes the test. Note the numbers...  In the event you didn't
write them down, all variables have been saved in the script SAVEVARS.
Press <CR> to end the program.
   Endtext
   Cursor Normal
   Accept "A1" To x
   Clear
Endproc




;-----------------------------------------------------------------------------
;  Note: decimal.hrs, decimal.mins and decimal.secs are the total elapsed time
;        expressed in terms of each unit of measure (Generic Proc)
;-----------------------------------------------------------------------------
Proc calc.elapsed(date.started,ticks.started,date.ended,ticks.ended)
   Private  temp.hrs, temp.mins, temp.secs

   days          = date.ended - date.started
   decimal.secs  = (days*24*3600+(ticks.ended-ticks.started)/1000)
   decimal.mins  = decimal.secs / 60
   decimal.hrs   = decimal.secs / 3600
   temp.fraction = mod(decimal.hrs,1)
   temp.hrs      = int(decimal.hrs)
   temp.mins     = int(temp.fraction * 60)
   temp.secs     = temp.fraction * 3600 - temp.mins * 60
   display.hrs   = substr(format("W5,ez",temp.hrs),2,4)
   display.mins  = substr(format("W3,ez",temp.mins),2,2)
   display.secs  = substr(format("W6.2,ez",temp.secs),2,5)
   display.time  = display.hrs+":"+display.mins+":"+display.secs
Endproc


bench()
